// This module deals with converting the notification object into Json format, which might be useful for data exchange
use crate::notification::Notification;
use serde_json::{json, Value}; // json!() macro and Value type

impl Notification {
    // Actions, as json
    pub fn actions_json(&self) -> Value {
        if self.actions().is_empty() {
            json!({}) // Empty JSON object if no actions
        } else {
            serde_json::Value::Object(
                // Wrap the map into a serde_json::Value
                self.actions()
                    .iter()
                    .map(|(id, label)| {
                        (id.clone(), json!(label)) // Create key-value pairs: id -> label
                    })
                    .collect::<serde_json::Map<_, _>>(), // Collect into a serde_json::Map
            )
        }
    }

    // Default action, as json string
    pub fn default_action_json(&self) -> Value {
        match self.default_action() {
            None => serde_json::Value::from("None"),
            Some(actn) => serde_json::Value::String(actn.0),
        }
    }

    // Hints, as json
    pub fn hints_json(&self) -> Value {
        if self.hints().is_empty() {
            json!({}) // Empty JSON object if no hints
        } else {
            serde_json::Value::Object(
                self.hints()
                    .iter()
                    .map(|(key, value)| {
                        (key.clone(), json!(value.to_string())) // Convert hint value to string
                    })
                    .collect::<serde_json::Map<_, _>>(), // Collect into a serde_json::Map
            )
        }
    }

    // The notification as a json object
    pub fn json(&self) -> Value {
        // Initialize
        let mut notif_json: Value = json!({
            "app_name": &self.app_name(),
            "replace_id": &self.replace_id(),
            "icon": &self.icon(),
            "summary": &self.summary(),
            "body": &self.body(),
            "hints": &self.hints_json(),
            "expiration_timeout": self.expir_timeout(),
            "urgency": self.urgency(),
        });

        // Conditionally add the Actions fields
        if let Value::Object(ref mut map) = notif_json {
            if !self.actions().is_empty() {
                map.insert("actions".to_string(), self.actions_json());
                map.insert("default_action".to_string(), self.default_action_json());
            }
        }
        notif_json
    }
}
